import numpy as np
import matplotlib.pyplot as plt
from Scientific.N import *
from Scientific.IO.NetCDF import *
from pylab import *
from scipy import interpolate as ip
from matplotlib import ticker
from mpl_toolkits.axes_grid1 import make_axes_locatable


font = {'family':'serif','serif':'serif','weight':'normal','size':'24'}
rc('font',**font)
rc('text',usetex=True)

#-------------------------------------------------------
#Import data
#-------------------------------------------------------
def loadNC(path,var_ind,time0=0,time1=0):
  file = NetCDFFile(path,"r")
  #i = variable number / variables are stored in inverse order (e.g 'var_0 = W', 'var_1 = V', 'var_2 = U', with U,V,W the displacement fields along x,y and Z directions) 

  dimNames = file.dimensions.keys()
  dimVals = file.dimensions.values()
  var_name = file.variables.keys()
  
  tmp = file.variables[var_name[var_ind]]
  varShape = tmp.shape
  
  # By default the 5 dimensions exist and some of them can be empty. Dimensions are stored as following : [t,c,z,y,x]
  # - x,y, z are spatial dimensions (z is empty if 2D case, and y is empty if 1D case)
  # - c (color channel) is never used (empty)
  # - t (time) corresponds to the sequence of treated image (empty if only one couple of picture)
  # To remove empty dimensions the following
  var = np.squeeze(tmp[time0:time1,:,:,:,:])

  file.close()

  return var


#-------------------------------------------------------
#Strain calcul
#-------------------------------------------------------
#win = Correlation ZOI size (assumed squared)
#U,V or W = Var[i]
def strain(gridX,gridY,U,V,gridZ=0,W=0):
  E = {}
  if W == 0:
    E['xx'] = np.zeros([U.shape[0],U.shape[1]])
    E['yy'] = np.zeros([U.shape[0],U.shape[1]])
    E['xy'] = np.zeros([U.shape[0],U.shape[1]])
    E['yx'] = np.zeros([U.shape[0],U.shape[1]])
    [E['xy'],E['xx']] = np.gradient(U,gridX,gridY)
    [E['yy'],E['yx']] = np.gradient(V,gridX,gridY)
  else:
    E['xx'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['yy'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['zz'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['xy'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['xz'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['yx'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['yz'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['zx'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    E['zy'] = np.zeros([U.shape[0],U.shape[1],U.shape[2]])
    [E['xy'],E['xx'],E['xz']] = np.gradient(U,gridX,gridY,gridZ)
    [E['yy'],E['yx'],E['yz']] = np.gradient(V,gridX,gridY,gridZ)
    [E['zy'],E['zx'],E['zz']] = np.gradient(W,gridX,gridY,gridZ)
  return E


#-------------------------------------------------------
#Plot 2D
#-------------------------------------------------------
# Displacement field ("mode.nc") superimposed on picture ("deform.nc"), using grid node ("node.nc")
def plot2d(picture,field, gridX, gridY, title):
  
  
  ax1 = plt.subplot(1,1,1)
  
  mini = field[:,:].mean() - 2*field[:,:].std()
  maxi = field[:,:].mean() + 2*field[:,:].std()
  
  plt.imshow(picture, origin='upper', interpolation='bicubic', extent=(gridX.min(), gridX.max(), gridY.min(), gridY.max()),cmap=cm.gray)
  ax = plt.imshow(field, origin='upper', interpolation='bicubic', extent=(gridX.min(), gridX.max(), gridY.min(), gridY.max()), cmap=cm.PuBu)
  
  plt.title(title)
  
  #colorbar
  divider = make_axes_locatable(plt.gca())
  cax = divider.append_axes("bottom", "5%", pad="3%")
  bar = plt.colorbar(ax, cax=cax, orientation='horizontal',format='%2.2f')
  tick_locator = ticker.MaxNLocator(nbins=6)
  bar.locator = tick_locator
  bar.update_ticks()
  
  #limits
  bar.set_clim([mini, maxi])
  ax1.set_ylim([gridY.min(), gridY.max()])
  ax1.set_xlim([gridX.min(), gridX.max()])
  
  #grid
  lineH = ax1.hlines(gridY[:,0],gridX[0,0],gridX[0,-1])
  setp(lineH, color='k', linewidth=0.1)
  lineV = ax1.vlines(gridX[0,:],gridY[0,0],gridY[-1,0])
  setp(lineV, color='k', linewidth=0.1)
  
  ax1.set_xticks(())
  ax1.set_yticks(())
  plt.savefig(title+'.png',dpi=300)
  plt.show()
  plt.clf()
  
  
  
#-------------------------------------------------------
#--------------------PROGRAM----------------------------
path = './Correlation/Bodelot08_modes.nc'
time = 20
V = loadNC(path,0,time,time+1)
U = loadNC(path,1,time,time+1)

path = './Correlation/Bodelot08_nodes.nc'
gridY = loadNC(path,0,0,1)
gridX = loadNC(path,1,0,1)
dx = np.gradient(gridX)[1]
dy = np.gradient(gridY)[0]
  
path = './Correlation/Bodelot08_mask_modes.nc'
msk = loadNC(path,0,0,1)
msk = np.abs(msk-1)

image = plt.imread('./img/frame.png')
image = image[::-1,:]

Umask = np.ma.masked_array(U,msk)
Vmask = np.ma.masked_array(V,msk)

E = strain(dx,dy,Umask,Vmask)

#field transformation
dat = np.sqrt(E['xx']**2+E['yy']**2-E['yy']*E['xx']+3*(0.5*(E['xy']+E['xy']))**2)*100

plot2d(image, dat, gridX, gridY, r'$\varepsilon_{eq}$ $(\%)$')
#-------------------------------------------------------
#-------------------------------------------------------